@@ -1,9 +1,8 @@
-/* Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements.  See the NOTICE file distributed with
- * this work for additional information regarding copyright ownership.
- * The ASF licenses this file to You under the Apache License, Version 2.0
- * (the "License"); you may not use this file except in compliance with
- * the License.  You may obtain a copy of the License at
+/* Copyright 2000-2004 The Apache Software Foundation
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
  *     http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -45,16 +44,13 @@ APR_DECLARE(apr_status_t) apr_file_dup(apr_file_t **new_file,
     (*new_file)->buffered = FALSE;
     (*new_file)->ungetchar = old_file->ungetchar;
 
-#if APR_HAS_THREADS
-    if (old_file->mutex) {
-        apr_thread_mutex_create(&((*new_file)->mutex),
-                                APR_THREAD_MUTEX_DEFAULT, p);
-    }
-#endif
-
     apr_pool_cleanup_register((*new_file)->pool, (void *)(*new_file), file_cleanup,
                         apr_pool_cleanup_null);
 
+    /* Create a pollset with room for one descriptor. */
+    /* ### check return codes */
+    (void) apr_pollset_create(&(*new_file)->pollset, 1, p, 0);
+
     return APR_SUCCESS;
 #endif /* !defined(_WIN32_WCE) */
 }
@@ -122,13 +118,6 @@ APR_DECLARE(apr_status_t) apr_file_dup2(apr_file_t *new_file,
     new_file->buffered = FALSE;
     new_file->ungetchar = old_file->ungetchar;
 
-#if APR_HAS_THREADS
-    if (old_file->mutex) {
-        apr_thread_mutex_create(&(new_file->mutex),
-                                APR_THREAD_MUTEX_DEFAULT, p);
-    }
-#endif
-
     return APR_SUCCESS;
 #endif /* !defined(_WIN32_WCE) */
 }
@@ -166,5 +155,10 @@ APR_DECLARE(apr_status_t) apr_file_setaside(apr_file_t **new_file,
     old_file->filehand = INVALID_HANDLE_VALUE;
     apr_pool_cleanup_kill(old_file->pool, (void *)old_file,
                           file_cleanup);
+
+    /* Create a pollset with room for one descriptor. */
+    /* ### check return codes */
+    (void) apr_pollset_create(&(*new_file)->pollset, 1, p, 0);
+
     return APR_SUCCESS;
 }
