@@ -1,60 +1,22 @@
-/* ====================================================================
- * The Apache Software License, Version 1.1
+/* Copyright 2000-2004 The Apache Software Foundation
  *
- * Copyright (c) 2000-2003 The Apache Software Foundation.  All rights
- * reserved.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Apache" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation.  For more
- * information on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 #include "apr.h"
 #include "apr_private.h"
 
+#include "apr_atomic.h"
 #include "apr_portable.h" /* for get_os_proc */
 #include "apr_strings.h"
 #include "apr_general.h"
@@ -473,6 +435,7 @@ struct apr_pool_t {
     apr_pool_t           *sibling;
     apr_pool_t          **ref;
     cleanup_t            *cleanups;
+    cleanup_t            *free_cleanups;
     apr_allocator_t      *allocator;
     struct process_chain *subprocesses;
     apr_abortfunc_t       abort_fn;
@@ -555,6 +518,13 @@ APR_DECLARE(apr_status_t) apr_pool_initialize(void)
 
     apr_pool_tag(global_pool, "apr_global_pool");
 
+    /* This has to happen here because mutexes might be backed by
+     * atomics.  It used to be snug and safe in apr_initialize().
+     */
+    if ((rv = apr_atomic_init(global_pool)) != APR_SUCCESS) {
+        return rv;
+    }
+
 #if APR_HAS_THREADS
     {
         apr_thread_mutex_t *mutex;
@@ -588,24 +558,21 @@ APR_DECLARE(void) apr_pool_terminate(void)
     global_allocator = NULL;
 }
 
-#ifdef NETWARE
-void netware_pool_proc_cleanup ()
-{
-    apr_pool_t *pool = global_pool->child;
-    apr_os_proc_t owner_proc = (apr_os_proc_t)getnlmhandle();
 
-    while (pool) {
-        if (pool->owner_proc == owner_proc) {
-            apr_pool_destroy (pool);
-            pool = global_pool->child;
-        }
-        else {
-            pool = pool->sibling;
-        }
-    }
-    return;
-}
-#endif /* defined(NETWARE) */
+/* Node list management helper macros; list_insert() inserts 'node'
+ * before 'point'. */
+#define list_insert(node, point) do {           \
+    node->ref = point->ref;                     \
+    *node->ref = node;                          \
+    node->next = point;                         \
+    point->ref = &node->next;                   \
+} while (0)
+
+/* list_remove() removes 'node' from its list. */
+#define list_remove(node) do {                  \
+    *node->ref = node->next;                    \
+    node->next->ref = node->ref;                \
+} while (0)
 
 /*
  * Memory allocation
@@ -630,8 +597,7 @@ APR_DECLARE(void *) apr_palloc(apr_pool_t *pool, apr_size_t size)
 
     node = active->next;
     if (size < (apr_size_t)(node->endp - node->first_avail)) {
-        *node->ref = node->next;
-        node->next->ref = node->ref;
+        list_remove(node);
     }
     else {
         if ((node = allocator_alloc(pool->allocator, size)) == NULL) {
@@ -647,10 +613,7 @@ APR_DECLARE(void *) apr_palloc(apr_pool_t *pool, apr_size_t size)
     mem = node->first_avail;
     node->first_avail += size;
 
-    node->ref = active->ref;
-    *node->ref = node;
-    node->next = active;
-    active->ref = &node->next;
+    list_insert(node, active);
 
     pool->active = node;
 
@@ -667,13 +630,8 @@ APR_DECLARE(void *) apr_palloc(apr_pool_t *pool, apr_size_t size)
     }
     while (free_index < node->free_index);
 
-    *active->ref = active->next;
-    active->next->ref = active->ref;
-
-    active->ref = node->ref;
-    *active->ref = active;
-    active->next = node;
-    node->ref = &active->next;
+    list_remove(active);
+    list_insert(active, node);
 
     return mem;
 }
@@ -717,6 +675,7 @@ APR_DECLARE(void) apr_pool_clear(apr_pool_t *pool)
     /* Run cleanups */
     run_cleanups(&pool->cleanups);
     pool->cleanups = NULL;
+    pool->free_cleanups = NULL;
 
     /* Free subprocesses */
     free_proc_chain(pool->subprocesses);
@@ -844,6 +803,7 @@ APR_DECLARE(apr_status_t) apr_pool_create_ex(apr_pool_t **newpool,
     pool->abort_fn = abort_fn;
     pool->child = NULL;
     pool->cleanups = NULL;
+    pool->free_cleanups = NULL;
     pool->subprocesses = NULL;
     pool->user_data = NULL;
     pool->tag = NULL;
@@ -936,13 +896,9 @@ static int psprintf_flush(apr_vformatter_buff_t *vbuff)
     node = active->next;
     if (!ps->got_a_new_node
         && size < (apr_size_t)(node->endp - node->first_avail)) {
-        *node->ref = node->next;
-        node->next->ref = node->ref;
 
-        node->ref = active->ref;
-        *node->ref = node;
-        node->next = active;
-        active->ref = &node->next;
+        list_remove(node);
+        list_insert(node, active);
 
         node->free_index = 0;
 
@@ -959,13 +915,8 @@ static int psprintf_flush(apr_vformatter_buff_t *vbuff)
             }
             while (free_index < node->free_index);
 
-            *active->ref = active->next;
-            active->next->ref = active->ref;
-
-            active->ref = node->ref;
-            *active->ref = active;
-            active->next = node;
-            node->ref = &active->next;
+            list_remove(active);
+            list_insert(active, node);
         }
 
         node = pool->active;
@@ -1050,10 +1001,7 @@ APR_DECLARE(char *) apr_pvsprintf(apr_pool_t *pool, const char *fmt, va_list ap)
 
     node->free_index = 0;
 
-    node->ref = active->ref;
-    *node->ref = node;
-    node->next = active;
-    active->ref = &node->next;
+    list_insert(node, active);
 
     pool->active = node;
 
@@ -1071,13 +1019,8 @@ APR_DECLARE(char *) apr_pvsprintf(apr_pool_t *pool, const char *fmt, va_list ap)
     }
     while (free_index < node->free_index);
 
-    *active->ref = active->next;
-    active->next->ref = active->ref;
-
-    active->ref = node->ref;
-    *active->ref = active;
-    active->next = node;
-    node->ref = &active->next;
+    list_remove(active);
+    list_insert(active, node);
 
     return strp;
 }
@@ -1266,6 +1209,13 @@ APR_DECLARE(apr_status_t) apr_pool_initialize(void)
 
     apr_pools_initialized = 1;
 
+    /* This has to happen here because mutexes might be backed by
+     * atomics.  It used to be snug and safe in apr_initialize().
+     */
+    if ((rv = apr_atomic_init(global_pool)) != APR_SUCCESS) {
+        return rv;
+    }
+
 #if (APR_POOL_DEBUG & APR_POOL_DEBUG_VERBOSE_ALL)
     apr_file_open_stderr(&file_stderr, global_pool);
     if (file_stderr) {
@@ -1393,8 +1343,13 @@ static void pool_clear_debug(apr_pool_t *pool, const char *file_line)
 
     /* Run cleanups */
     run_cleanups(&pool->cleanups);
+    pool->free_cleanups = NULL;
     pool->cleanups = NULL;
 
+    /* If new child pools showed up, this is a reason to raise a flag */
+    if (pool->child)
+        abort();
+
     /* Free subprocesses */
     free_proc_chain(pool->subprocesses);
     pool->subprocesses = NULL;
@@ -1558,6 +1513,10 @@ APR_DECLARE(apr_status_t) apr_pool_create_ex_debug(apr_pool_t **newpool,
 #if APR_HAS_THREADS
     pool->owner = apr_os_thread_current();
 #endif /* APR_HAS_THREADS */
+#ifdef NETWARE
+    pool->owner_proc = (apr_os_proc_t)getnlmhandle();
+#endif /* defined(NETWARE) */
+
 
     if (parent == NULL || parent->allocator != allocator) {
 #if APR_HAS_THREADS
@@ -1751,6 +1710,25 @@ APR_DECLARE(void) apr_pool_lock(apr_pool_t *pool, int flag)
 
 #endif /* !APR_POOL_DEBUG */
 
+#ifdef NETWARE
+void netware_pool_proc_cleanup ()
+{
+    apr_pool_t *pool = global_pool->child;
+    apr_os_proc_t owner_proc = (apr_os_proc_t)getnlmhandle();
+
+    while (pool) {
+        if (pool->owner_proc == owner_proc) {
+            apr_pool_destroy (pool);
+            pool = global_pool->child;
+        }
+        else {
+            pool = pool->sibling;
+        }
+    }
+    return;
+}
+#endif /* defined(NETWARE) */
+
 
 /*
  * "Print" functions (common)
@@ -1784,6 +1762,13 @@ APR_DECLARE(apr_abortfunc_t) apr_pool_abort_get(apr_pool_t *pool)
 
 APR_DECLARE(apr_pool_t *) apr_pool_parent_get(apr_pool_t *pool)
 {
+#ifdef NETWARE
+    /* On NetWare, don't return the global_pool, return the application pool 
+       as the top most pool */
+    if (pool->parent == global_pool)
+        return NULL;
+    else
+#endif
     return pool->parent;
 }
 
@@ -1905,7 +1890,13 @@ APR_DECLARE(void) apr_pool_cleanup_register(apr_pool_t *p, const void *data,
 #endif /* APR_POOL_DEBUG */
 
     if (p != NULL) {
-        c = (cleanup_t *)apr_palloc(p, sizeof(cleanup_t));
+        if (p->free_cleanups) {
+            /* reuse a cleanup structure */
+            c = p->free_cleanups;
+            p->free_cleanups = c->next;
+        } else {
+            c = apr_palloc(p, sizeof(cleanup_t));
+        }
         c->data = data;
         c->plain_cleanup_fn = plain_cleanup_fn;
         c->child_cleanup_fn = child_cleanup_fn;
@@ -1931,6 +1922,9 @@ APR_DECLARE(void) apr_pool_cleanup_kill(apr_pool_t *p, const void *data,
     while (c) {
         if (c->data == data && c->plain_cleanup_fn == cleanup_fn) {
             *lastp = c->next;
+            /* move to freelist */
+            c->next = p->free_cleanups;
+            p->free_cleanups = c;
             break;
         }
 
@@ -2228,60 +2222,3 @@ APR_DECLARE(apr_status_t) apr_pool_create_ex(apr_pool_t **newpool,
 }
 
 #endif /* APR_POOL_DEBUG */
-
-/* Deprecated */
-APR_DECLARE(void) apr_allocator_set_max_free(apr_allocator_t *allocator,
-                                             apr_size_t size)
-{
-    apr_allocator_max_free_set(allocator, size);
-}
-
-/* Deprecated */
-APR_DECLARE(void) apr_pool_set_abort(apr_abortfunc_t abort_fn,
-                                     apr_pool_t *pool)
-{
-    apr_pool_abort_set(abort_fn, pool);
-}
-
-/* Deprecated */
-APR_DECLARE(apr_abortfunc_t) apr_pool_get_abort(apr_pool_t *pool)
-{
-    return apr_pool_abort_get(pool);
-}
-
-/* Deprecated */
-APR_DECLARE(apr_pool_t *) apr_pool_get_parent(apr_pool_t *pool)
-{
-    return apr_pool_parent_get(pool);
-}
-
-/* Deprecated */
-APR_DECLARE(void) apr_allocator_set_owner(apr_allocator_t *allocator,
-                                          apr_pool_t *pool)
-{
-    apr_allocator_owner_set(allocator, pool);
-}
-
-/* Deprecated */
-APR_DECLARE(apr_pool_t *) apr_allocator_get_owner(
-                                  apr_allocator_t *allocator)
-{
-    return apr_allocator_owner_get(allocator);
-}
-
-#if APR_HAS_THREADS
-/* Deprecated */
-APR_DECLARE(apr_thread_mutex_t *) apr_allocator_get_mutex(
-                                      apr_allocator_t *allocator)
-{
-    return apr_allocator_mutex_get(allocator);
-}
-
-/* Deprecated */
-APR_DECLARE(void) apr_allocator_set_mutex(apr_allocator_t *allocator,
-                                          apr_thread_mutex_t *mutex)
-{
-    apr_allocator_mutex_set(allocator, mutex);
-}
-#endif /* APR_HAS_THREADS */
-
