@@ -1,9 +1,8 @@
-/* Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements.  See the NOTICE file distributed with
- * this work for additional information regarding copyright ownership.
- * The ASF licenses this file to You under the Apache License, Version 2.0
- * (the "License"); you may not use this file except in compliance with
- * the License.  You may obtain a copy of the License at
+/* Copyright 2000-2004 The Apache Software Foundation
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
  *     http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -32,10 +31,6 @@
 #include <process.h>
 #endif
 
-#ifndef LOGON32_LOGON_NETWORK
-#define LOGON32_LOGON_NETWORK 3
-#endif
-
 #ifdef _WIN32_WCE
 #ifndef DETACHED_PROCESS
 #define DETACHED_PROCESS 0
@@ -250,14 +245,14 @@ static char *apr_caret_escape_args(apr_pool_t *p, const char *str)
 APR_DECLARE(apr_status_t) apr_procattr_child_errfn_set(apr_procattr_t *attr,
                                                        apr_child_errfn_t *errfn)
 {
-    attr->errfn = errfn;
+    /* won't ever be called on this platform, so don't save the function pointer */
     return APR_SUCCESS;
 }
 
 APR_DECLARE(apr_status_t) apr_procattr_error_check_set(apr_procattr_t *attr,
                                                        apr_int32_t chk)
 {
-    attr->errchk = chk;
+    /* won't ever be used on this platform, so don't save the flag */
     return APR_SUCCESS;
 }
 
@@ -268,47 +263,6 @@ APR_DECLARE(apr_status_t) apr_procattr_addrspace_set(apr_procattr_t *attr,
     return APR_SUCCESS;
 }
 
-#if APR_HAS_UNICODE_FS && !defined(_WIN32_WCE)
-
-/* Used only for the NT code path, a critical section is the fastest
- * implementation available.
- */
-static CRITICAL_SECTION proc_lock;
-
-static apr_status_t threadproc_global_cleanup(void *ignored)
-{
-    DeleteCriticalSection(&proc_lock);
-    return APR_SUCCESS;
-}
-
-/* Called from apr_initialize, we need a critical section to handle
- * the pipe inheritance on win32.  This will mutex any process create
- * so as we change our inherited pipes, we prevent another process from
- * also inheriting those alternate handles, and prevent the other process
- * from failing to inherit our standard handles.
- */
-apr_status_t apr_threadproc_init(apr_pool_t *pool)
-{
-    IF_WIN_OS_IS_UNICODE
-    {
-        InitializeCriticalSection(&proc_lock);
-        /* register the cleanup */
-        apr_pool_cleanup_register(pool, &proc_lock,
-                                  threadproc_global_cleanup,
-                                  apr_pool_cleanup_null);
-    }
-    return APR_SUCCESS;
-}
-
-#else /* !APR_HAS_UNICODE_FS || defined(_WIN32_WCE) */
-
-apr_status_t apr_threadproc_init(apr_pool_t *pool)
-{
-    return APR_SUCCESS;
-}
-
-#endif
-
 APR_DECLARE(apr_status_t) apr_proc_create(apr_proc_t *new,
                                           const char *progname,
                                           const char * const *args,
@@ -357,12 +311,6 @@ APR_DECLARE(apr_status_t) apr_proc_create(apr_proc_t *new,
         char *fullpath = NULL;
         if ((rv = apr_filepath_merge(&fullpath, attr->currdir, progname, 
                                      APR_FILEPATH_NATIVE, pool)) != APR_SUCCESS) {
-            if (attr->errfn) {
-                attr->errfn(pool, rv, 
-                            apr_pstrcat(pool, "filepath_merge failed.", 
-                                        " currdir: ", attr->currdir, 
-                                        " progname: ", progname,NULL));
-            }
             return rv;
         }
         progname = fullpath;
@@ -392,7 +340,7 @@ APR_DECLARE(apr_status_t) apr_proc_create(apr_proc_t *new,
     /* Handle the args, seperate from argv0 */
     cmdline = "";
     for (i = 1; args && args[i]; ++i) {
-        if (has_space(args[i]) || !args[i][0]) {
+        if (has_space(args[i])) {
             cmdline = apr_pstrcat(pool, cmdline, " \"", args[i], "\"", NULL);
         }
         else {
@@ -404,9 +352,6 @@ APR_DECLARE(apr_status_t) apr_proc_create(apr_proc_t *new,
     if (attr->cmdtype == APR_SHELLCMD || attr->cmdtype == APR_SHELLCMD_ENV) {
         char *shellcmd = getenv("COMSPEC");
         if (!shellcmd) {
-            if (attr->errfn) {
-                attr->errfn(pool, APR_EINVAL, "COMSPEC envar is not set");
-            }
             return APR_EINVAL;
         }
         if (shellcmd[0] == '"') {
@@ -431,9 +376,6 @@ APR_DECLARE(apr_status_t) apr_proc_create(apr_proc_t *new,
     else 
 #endif
     {
-#if defined(_WIN32_WCE)
-        {
-#else
         /* Win32 is _different_ than unix.  While unix will find the given
          * program since it's already chdir'ed, Win32 cannot since the parent
          * attempts to open the program with it's own path.
@@ -446,9 +388,6 @@ APR_DECLARE(apr_status_t) apr_proc_create(apr_proc_t *new,
         {
             char *shellcmd = getenv("COMSPEC");
             if (!shellcmd) {
-                if (attr->errfn) {
-                    attr->errfn(pool, APR_EINVAL, "COMSPEC envar is not set");
-                }
                 return APR_EINVAL;
             }
             if (shellcmd[0] == '"') {
@@ -491,7 +430,6 @@ APR_DECLARE(apr_status_t) apr_proc_create(apr_proc_t *new,
             }
         }
         else {
-#endif
             /* A simple command we are directly invoking.  Do not pass
              * the first arg to CreateProc() for APR_PROGRAM_PATH
              * invocation, since it would need to be a literal and
@@ -541,12 +479,6 @@ APR_DECLARE(apr_status_t) apr_proc_create(apr_proc_t *new,
                 if ((rv = apr_conv_utf8_to_ucs2(env[i], &in, 
                                                 pNext, &iEnvBlockLen)) 
                         != APR_SUCCESS) {
-                    if (attr->errfn) {
-                        attr->errfn(pool, rv, 
-                                    apr_pstrcat(pool, 
-                                                "utf8 to ucs2 conversion failed" 
-                                                " on this string: ", env[i], NULL));
-                    }
                     return rv;
                 }
                 pNext = wcschr(pNext, L'\0') + 1;
@@ -583,9 +515,6 @@ APR_DECLARE(apr_status_t) apr_proc_create(apr_proc_t *new,
     IF_WIN_OS_IS_UNICODE
     {
         STARTUPINFOW si;
-        DWORD stdin_reset = 0;
-        DWORD stdout_reset = 0;
-        DWORD stderr_reset = 0;
         apr_wchar_t *wprg = NULL;
         apr_wchar_t *wcmd = NULL;
         apr_wchar_t *wcwd = NULL;
@@ -596,12 +525,6 @@ APR_DECLARE(apr_status_t) apr_proc_create(apr_proc_t *new,
             wprg = apr_palloc(pool, nwprg * sizeof(wprg[0]));
             if ((rv = apr_conv_utf8_to_ucs2(progname, &nprg, wprg, &nwprg))
                    != APR_SUCCESS) {
-                if (attr->errfn) {
-                    attr->errfn(pool, rv, 
-                                apr_pstrcat(pool, 
-                                            "utf8 to ucs2 conversion failed" 
-                                            " on progname: ", progname, NULL));
-                }
                 return rv;
             }
         }
@@ -612,12 +535,6 @@ APR_DECLARE(apr_status_t) apr_proc_create(apr_proc_t *new,
             wcmd = apr_palloc(pool, nwcmd * sizeof(wcmd[0]));
             if ((rv = apr_conv_utf8_to_ucs2(cmdline, &ncmd, wcmd, &nwcmd))
                     != APR_SUCCESS) {
-                if (attr->errfn) {
-                    attr->errfn(pool, rv, 
-                                apr_pstrcat(pool, 
-                                            "utf8 to ucs2 conversion failed" 
-                                            " on cmdline: ", cmdline, NULL));
-                }
                 return rv;
             }
         }
@@ -630,12 +547,6 @@ APR_DECLARE(apr_status_t) apr_proc_create(apr_proc_t *new,
             if ((rv = apr_conv_utf8_to_ucs2(attr->currdir, &ncwd, 
                                             wcwd, &nwcwd))
                     != APR_SUCCESS) {
-                if (attr->errfn) {
-                    attr->errfn(pool, rv, 
-                                apr_pstrcat(pool, 
-                                            "utf8 to ucs2 conversion failed" 
-                                            " on currdir: ", attr->currdir, NULL));
-                }
                 return rv;
             }
         }
@@ -649,58 +560,23 @@ APR_DECLARE(apr_status_t) apr_proc_create(apr_proc_t *new,
         }
 
 #ifndef _WIN32_WCE
-        /* LOCK CRITICAL SECTION 
-         * before we begin to manipulate the inherited handles
-         */
-        EnterCriticalSection(&proc_lock);
-
         if ((attr->child_in && attr->child_in->filehand)
             || (attr->child_out && attr->child_out->filehand)
             || (attr->child_err && attr->child_err->filehand))
         {
             si.dwFlags |= STARTF_USESTDHANDLES;
 
-            si.hStdInput = GetStdHandle(STD_INPUT_HANDLE);
-            if (attr->child_in && attr->child_in->filehand)
-            {
-                if (GetHandleInformation(si.hStdInput,
-                                         &stdin_reset)
-                        && (stdin_reset &= HANDLE_FLAG_INHERIT))
-                    SetHandleInformation(si.hStdInput,
-                                         HANDLE_FLAG_INHERIT, 0);
+            si.hStdInput = (attr->child_in) 
+                              ? attr->child_in->filehand
+                              : INVALID_HANDLE_VALUE;
 
-                si.hStdInput = attr->child_in->filehand;
-                SetHandleInformation(si.hStdInput, HANDLE_FLAG_INHERIT,
-                                                   HANDLE_FLAG_INHERIT);
-            }
-            
-            si.hStdOutput = GetStdHandle(STD_OUTPUT_HANDLE);
-            if (attr->child_out && attr->child_out->filehand)
-            {
-                if (GetHandleInformation(si.hStdOutput,
-                                         &stdout_reset)
-                        && (stdout_reset &= HANDLE_FLAG_INHERIT))
-                    SetHandleInformation(si.hStdOutput,
-                                         HANDLE_FLAG_INHERIT, 0);
+            si.hStdOutput = (attr->child_out)
+                              ? attr->child_out->filehand
+                              : INVALID_HANDLE_VALUE;
 
-                si.hStdOutput = attr->child_out->filehand;
-                SetHandleInformation(si.hStdOutput, HANDLE_FLAG_INHERIT,
-                                                    HANDLE_FLAG_INHERIT);
-            }
-
-            si.hStdError = GetStdHandle(STD_ERROR_HANDLE);
-            if (attr->child_err && attr->child_err->filehand)
-            {
-                if (GetHandleInformation(si.hStdError,
-                                         &stderr_reset)
-                        && (stderr_reset &= HANDLE_FLAG_INHERIT))
-                    SetHandleInformation(si.hStdError,
-                                         HANDLE_FLAG_INHERIT, 0);
-
-                si.hStdError = attr->child_err->filehand;
-                SetHandleInformation(si.hStdError, HANDLE_FLAG_INHERIT,
-                                                   HANDLE_FLAG_INHERIT);
-            }
+            si.hStdError = (attr->child_err)
+                              ? attr->child_err->filehand
+                              : INVALID_HANDLE_VALUE;
         }
         rv = CreateProcessW(wprg, wcmd,        /* Executable & Command line */
                             NULL, NULL,        /* Proc & thread security attributes */
@@ -709,29 +585,7 @@ APR_DECLARE(apr_status_t) apr_proc_create(apr_proc_t *new,
                             pEnvBlock,         /* Environment block */
                             wcwd,              /* Current directory name */
                             &si, &pi);
-
-        if ((attr->child_in && attr->child_in->filehand)
-            || (attr->child_out && attr->child_out->filehand)
-            || (attr->child_err && attr->child_err->filehand))
-        {
-            if (stdin_reset)
-                SetHandleInformation(GetStdHandle(STD_INPUT_HANDLE),
-                                     stdin_reset, stdin_reset);
-
-            if (stdout_reset)
-                SetHandleInformation(GetStdHandle(STD_OUTPUT_HANDLE),
-                                     stdout_reset, stdout_reset);
-
-            if (stderr_reset)
-                SetHandleInformation(GetStdHandle(STD_ERROR_HANDLE),
-                                     stderr_reset, stderr_reset);
-        }
-        /* RELEASE CRITICAL SECTION 
-         * The state of the inherited handles has been restored.
-         */
-        LeaveCriticalSection(&proc_lock);
-
-#else /* defined(_WIN32_WCE) */
+#else
         rv = CreateProcessW(wprg, wcmd,        /* Executable & Command line */
                             NULL, NULL,        /* Proc & thread security attributes */
                             FALSE,             /* must be 0 */
@@ -763,15 +617,15 @@ APR_DECLARE(apr_status_t) apr_proc_create(apr_proc_t *new,
 
             si.hStdInput = (attr->child_in) 
                               ? attr->child_in->filehand
-                              : GetStdHandle(STD_INPUT_HANDLE);
+                              : INVALID_HANDLE_VALUE;
 
             si.hStdOutput = (attr->child_out)
                               ? attr->child_out->filehand
-                              : GetStdHandle(STD_OUTPUT_HANDLE);
+                              : INVALID_HANDLE_VALUE;
 
             si.hStdError = (attr->child_err)
                               ? attr->child_err->filehand
-                              : GetStdHandle(STD_ERROR_HANDLE);
+                              : INVALID_HANDLE_VALUE;
         }
 
         rv = CreateProcessA(progname, cmdline, /* Command line */
